@@ -9,7 +9,7 @@ package WWW::Shopify::Exception;
 use Data::Dumper;
 use overload 
 	'fallback' => 1,
-	'""' => sub { return "Error: " . Dumper($_[0]->{'error'}) };
+	'""' => sub { return "Error: " . Dumper($_[0]->error, $_[0]->stack) };
 # Generic constructor; class is blessed with the package that new specifies, and contains a hash specified inside the parentheses of a new call.
 # Example: new WWW::Shopify::Exception('try' => 'catch'); $_[0] is 'WWW::Shopify::Exception', $_[1] is {'try' => 'catch'}.
 # The object will be of type WWW::Shopify::Exception, and have the contents of {'try' => 'catch'}.
